# -*- coding: utf-8 -*-
"""
Panda Physics - the panda3D simple physics engine

Level: BEGINNER

We're going now to start to discover what panda3d got to offer about this subject and how affect collision handling.

NOTE If you won't find here some line of code explained, probably you missed it in the previous steps - if you don't find there as well though, or still isn't clear for you, browse at http://www.panda3d.org/phpbb2/viewtopic.php?t=7918 and post your issue to the thread.
"""
import random
from direct.showbase.DirectObject import DirectObject
from pandac.PandaModules import *
from direct.interval.IntervalGlobal import *

loadPrcFileData("", """sync-video 0
"""
)
import direct.directbase.DirectStart
#** snippet support routines - not concerning the tutorial part
import snipstuff

#=========================================================================
# Scenographic stuff
#=========================================================================

base.cam.setPos(0,-30,10)
base.cam.lookAt((0,0,4))

snipstuff.info.append("Physics Glitch")
snipstuff.info.append("showing off main node - physics node loss of sync")
snipstuff.info.append("Try to push the big frowneys to not see centered the smiley avatar anymore.\n\nn=synch the nodes\nwasd=move the avatar around\nSPACE=avatar jump")
snipstuff.infotext['message']='Note the camera synch loss pushing the big spheres.\nNote: the camera lookAt() the main smiley node.'
snipstuff.info_show()

#=========================================================================
# Main

base.cTrav=CollisionTraverser()
base.enableParticles()
collisionHandler = PhysicsCollisionHandler()

#**
cp = CollisionPlane(Plane(Vec3(0, 0, 1), Point3(0, 0, 0)))
planeNP = base.render.attachNewNode(CollisionNode('planecnode'))
planeNP.node().addSolid(cp)
planeNP.show()

#**
globalforcesFN=ForceNode('world-forces')
globalforcesFNP=base.render.attachNewNode(globalforcesFN)
globalforcesGravity=LinearVectorForce(0,0,-9.81)
globalforcesFN.addForce(globalforcesGravity)
base.physicsMgr.addLinearForce(globalforcesGravity)

#** Inside this function we'll load a model and assign a collide ball to it, suitable to collide with everything interacting in the physics environment. I put it in a function because I'll going to call it several times. It will return at last the topmost nodepath of its structure so that we may drive each ball around.
def phyball_dispenser(modelname, scale=1.):
  # first off we gotta define the topmost node that should be a PandaNode wrapped into a nodepath - this is mandatory cos if we try to directly use the  Actornode defined below, we'll face inexpected behavior manipulating the object.
  ballNP=NodePath(PandaNode("phisicsball"))
  ballAN=ActorNode("ballactnode")
  ballANP=ballNP.attachNewNode(ballAN)
  ballmodel=loader.loadModel(modelname)
  ballmodel.reparentTo(ballANP)
  ballmodel.setScale(scale)

  ballCollider = ballANP.attachNewNode(CollisionNode('ballcnode'))
  ballCollider.node().addSolid(CollisionSphere(0,0,0, 1*scale))
  base.physicsMgr.attachPhysicalNode(ballAN)
  collisionHandler.addCollider(ballCollider, ballANP)
  base.cTrav.addCollider(ballCollider, collisionHandler)
  # now the physic ball is ready to exit off the dispenser - Note we reparent it to render by default
  ballNP.reparentTo(base.render)
  return ballNP

#** let's call our dispenser made above to create a physics smiley
smiley=phyball_dispenser('smiley')

#** here is where will be produced the falling objects rain
def objectrain(howmany):
  def newfroney():
    phyball=phyball_dispenser('frowney', 2)
    pos=(random.uniform(-1,1), random.uniform(-.5,.5), 30)
    phyball.setPos(pos)
  ivals=[]
  for i in range(howmany):
    ivals.append(Func(newfroney))
    ivals.append(Wait(.3))
  Sequence(*ivals).start()
snipstuff.DO.accept('r', objectrain, [30])

#** Now we're ready to activate the avatar steering and go.
steering=snipstuff.avatar_steer(smiley)
steering.start()

#** syncronize smiley topmost nodepath with physics actornode nodepath
def nsync():
  _smiley_po=smiley.getChild(0).node().getPhysicsObject()
  smiley.setPos(smiley.getPos()+_smiley_po.getPosition())
  _smiley_po.setPosition((0,0,0))
snipstuff.DO.accept('n', nsync)

#** let's start the show
run()
